@@ -1,4 +1,4 @@
-// $Id: CatapultConfigDlg.h,v 1.3 2004/04/18 08:43:19 h_oudejans Exp $
+// $Id: CatapultConfigDlg.h 9266 2009-02-26 23:36:58Z mthuurne $
 // CatapultConfigDlg.h: interface for the CatapultConfigDlg class.
 //
 //////////////////////////////////////////////////////////////////////
@@ -10,20 +10,24 @@
 #pragma once
 #endif // _MSC_VER > 1000
 
-class CatapultConfigDlg : public wxDialog  
+class openMSXController;
+
+class CatapultConfigDlg : public wxDialog
 {
 	public:
 		void OnBrowseExec (wxCommandEvent& event);
 		void OnBrowseShare (wxCommandEvent& event);
 		void OnCancel(wxCommandEvent& event);
 		void OnOk(wxCommandEvent& event);
-		CatapultConfigDlg(wxWindow * parent = NULL);
+		CatapultConfigDlg(wxWindow * parent, openMSXController* controller);
 		virtual ~CatapultConfigDlg();
 
 	private:
 		wxTextCtrl * m_ExecPath;
 		wxTextCtrl * m_SharePath;
-
+		wxCheckBox * m_ConfigCheck;
+		wxWindow * m_parent;
+                openMSXController * m_controller;
 
 		DECLARE_CLASS(CatapultConfigDlg)
 		// any class wishing to process wxWindows events must use this macro
