@@ -1,4 +1,4 @@
-// $Id: MiscControlPage.cpp 9266 2009-02-26 23:36:58Z mthuurne $
+// $Id: MiscControlPage.cpp 10815 2009-11-22 17:03:06Z mthuurne $
 // MiscControlPage.cpp: implementation of the MiscControlPage class.
 //
 //////////////////////////////////////////////////////////////////////
@@ -131,7 +131,7 @@ void MiscControlPage::FillInitialJoystickPortValues ()
 	wxString current;
 	int pos;
 
-#ifndef __BSD__
+#if wxUSE_JOYSTICK
 	wxJoystick joy(wxJOYSTICK1);
 	wxString temp;
 #endif
@@ -149,7 +149,7 @@ void MiscControlPage::FillInitialJoystickPortValues ()
 		box[i]->Append(wxT("keyjoystick1"));
 		box[i]->Append(wxT("keyjoystick2"));
 
-#ifndef __BSD__
+#if wxUSE_JOYSTICK
 		for (int j=1;j<=joy.GetNumberJoysticks();j++) {
 			temp.sprintf(wxT("joystick%d"),j);
 			box[i]->Append(temp);
