@@ -1,4 +1,4 @@
-// $Id: SessionPage.cpp,v 1.63 2005/06/05 19:38:59 h_oudejans Exp $
+// $Id: SessionPage.cpp 9266 2009-02-26 23:36:58Z mthuurne $
 // SessionPage.cpp: implementation of the SessionPage class.
 //
 //////////////////////////////////////////////////////////////////////
@@ -48,7 +48,8 @@ enum
 	Cas_Browse_File,
 	Cas_Eject,
 	Cas_Rewind,
-	Cas_ForcePlay
+	Cas_MotorControl,
+	Cas_AutoCreateFile
 };
 
 IMPLEMENT_CLASS(SessionPage, wxPanel)
@@ -58,6 +59,8 @@ BEGIN_EVENT_TABLE(SessionPage, wxPanel)
 	EVT_COMBOBOX(XRCID("CartAContents"),SessionPage::OnClickCartACombo)
 	EVT_COMBOBOX(XRCID("CartBContents"),SessionPage::OnClickCartBCombo)
 	EVT_COMBOBOX(XRCID("CassetteContents"),SessionPage::OnClickCassetteCombo)
+	EVT_TOGGLEBUTTON(XRCID("PlayButton"),SessionPage::OnModePlay)
+	EVT_TOGGLEBUTTON(XRCID("RecordButton"),SessionPage::OnModeRecord)
 	EVT_BUTTON(XRCID("DiskA_Button"),SessionPage::OnClickDiskMenu)
 	EVT_BUTTON(XRCID("DiskB_Button"),SessionPage::OnClickDiskMenu)
 	EVT_BUTTON(XRCID("CartA_Button"),SessionPage::OnClickCartMenu)
@@ -68,13 +71,12 @@ BEGIN_EVENT_TABLE(SessionPage, wxPanel)
 	EVT_BUTTON(XRCID("BrowseCartA"),SessionPage::OnBrowseCartA)
 	EVT_BUTTON(XRCID("BrowseCartB"),SessionPage::OnBrowseCartB)
 	EVT_BUTTON(XRCID("BrowseCassette"),SessionPage::OnBrowseCassette)
-	EVT_BUTTON(XRCID("ClearDiskA"),SessionPage::OnClearDiskA)
-	EVT_BUTTON(XRCID("ClearDiskB"),SessionPage::OnClearDiskB)
-	EVT_BUTTON(XRCID("ClearCartA"),SessionPage::OnClearCartA)
-	EVT_BUTTON(XRCID("ClearCartB"),SessionPage::OnClearCartB)
+	EVT_BUTTON(XRCID("ClearDiskA"),SessionPage::OnEjectDiskA)
+	EVT_BUTTON(XRCID("ClearDiskB"),SessionPage::OnEjectDiskB)
+	EVT_BUTTON(XRCID("ClearCartA"),SessionPage::OnEjectCartA)
+	EVT_BUTTON(XRCID("ClearCartB"),SessionPage::OnEjectCartB)
 	EVT_BUTTON(XRCID("ClearCassette"),SessionPage::OnClearCassette)
 	EVT_BUTTON(XRCID("RewindButton"),SessionPage::OnRewind)
-	EVT_TOGGLEBUTTON(XRCID("ForcePlayButton"),SessionPage::OnForcePlay)
 	EVT_TEXT(XRCID("DiskAContents"),SessionPage::OnChangeDiskAContents)
 	EVT_TEXT(XRCID("DiskBContents"),SessionPage::OnChangeDiskBContents)
 	EVT_TEXT(XRCID("CartAContents"),SessionPage::OnChangeCartAContents)
@@ -92,8 +94,8 @@ BEGIN_EVENT_TABLE(SessionPage, wxPanel)
 	EVT_MENU(Cas_Browse_File,SessionPage::OnBrowseCassette)
 	EVT_MENU(Cas_Eject,SessionPage::OnClearCassette)
 	EVT_MENU(Cas_Rewind,SessionPage::OnRewind)
-	EVT_MENU(Cas_ForcePlay,SessionPage::OnForcePlayByMenu)
-	EVT_MENU_CLOSE(SessionPage::OnCloseMenu)
+	EVT_MENU(Cas_MotorControl,SessionPage::OnMotorControl)
+	EVT_MENU(Cas_AutoCreateFile, SessionPage::OnAutoCassettefile)
 END_EVENT_TABLE()
 
 	//////////////////////////////////////////////////////////////////////
@@ -126,15 +128,17 @@ SessionPage::SessionPage(wxWindow * parent, openMSXController * controller)
 	m_casMenu->Append(Cas_Browse_File,wxT("Browse cassette image"),wxT(""),wxITEM_NORMAL);
 	m_casMenu->Append(Cas_Eject,wxT("Eject cassette"),wxT(""),wxITEM_NORMAL);
 	m_casMenu->Append(Cas_Rewind,wxT("Rewind cassette"),wxT(""),wxITEM_NORMAL);
-	m_casMenu->Append(Cas_ForcePlay,wxT("Force play"),wxT(""),wxITEM_CHECK);
-
+	m_casMenu->Append(Cas_MotorControl,wxT("Motor control"),wxT(""),wxITEM_CHECK);
+	m_casMenu->Append(Cas_AutoCreateFile,wxT("Auto create Cassette file for recording"),wxT(""),wxITEM_CHECK);
 
 	m_diskA = new mediaInfo(m_diskMenu[0]);
 	m_diskB = new mediaInfo(m_diskMenu[1]);
 	m_cartA = new mediaInfo(m_cartMenu[0]);
 	m_cartB = new mediaInfo(m_cartMenu[1]);
 	m_cassette = new mediaInfo(m_casMenu);
-
+	m_diskA->deviceName = wxT("diska");
+	m_diskB->deviceName = wxT("diskb");
+	m_cassette->deviceName = wxT("cassetteplayer");
 	m_parent = (wxCatapultFrame *)parent->GetParent()->GetParent();
 	m_diskA->control = (wxComboBox *)FindWindowByName(wxT("DiskAContents"));
 	m_diskB->control = (wxComboBox *)FindWindowByName(wxT("DiskBContents"));
@@ -153,7 +157,8 @@ SessionPage::SessionPage(wxWindow * parent, openMSXController * controller)
 	m_clearCartA = (wxBitmapButton *)FindWindowByName(wxT("ClearCartA"));
 	m_clearCartB = (wxBitmapButton *)FindWindowByName(wxT("ClearCartB"));
 	m_clearCassette = (wxBitmapButton *)FindWindowByName(wxT("ClearCassette"));
-    m_forcePlayButton = (wxToggleButton *)FindWindowByName(wxT("ForcePlayButton"));
+	m_playButton = (wxToggleButton *)FindWindowByName(wxT("PlayButton"));
+	m_recordButton = (wxToggleButton *)FindWindowByName(wxT("RecordButton"));
 	m_rewindButton = (wxButton *)FindWindowByName(wxT("RewindButton"));
 	m_diskAButton = (wxButton *)FindWindowByName(wxT("DiskA_Button"));
 	m_diskBButton = (wxButton *)FindWindowByName(wxT("DiskB_Button"));
@@ -162,15 +167,30 @@ SessionPage::SessionPage(wxWindow * parent, openMSXController * controller)
 	m_cassetteButton = (wxButton *)FindWindowByName(wxT("CassetteButton"));
 	m_machineListLabel = (wxStaticText *)FindWindowByName(wxT("MachineListLabel"));
 	m_extensionListLabel = (wxStaticText *)FindWindowByName(wxT("ExtensionLabel"));
-	m_lastDiskA = wxT("");
-	m_lastDiskB = wxT("");
-	m_lastCassette = wxT("");
+	//SetupHardware(true,false); // No need to do this, it's done in wxCatapultFrm's constructor
 
-	SetupHardware(true);
+	m_cassettePortState = wxT("disabled");
+	m_romTypeDialog = new RomTypeDlg (wxGetTopLevelParent(this));
+	GetRomTypes();
+	m_ipsDialog = new IPSSelectionDlg (wxGetTopLevelParent(this));
+	m_diskA->control->SetDropTarget(new SessionDropTarget(m_diskA->control));
+	m_diskB->control->SetDropTarget(new SessionDropTarget(m_diskB->control));
+	m_cartA->control->SetDropTarget(new SessionDropTarget(m_cartA->control));
+	m_cartB->control->SetDropTarget(new SessionDropTarget(m_cartB->control));
+	m_cassette->control->SetDropTarget(new SessionDropTarget(m_cassette->control));
 
+	int autorecord;
+	ConfigurationData::instance()->GetParameter(ConfigurationData::CD_AUTORECORD, &autorecord);
+	m_cassetteAutoCreate = (autorecord == 1);
+	m_casInsertCommand = wxT("insert ");
+	m_motorControlOnCommand = wxT("motorcontrol on");
+	m_motorControlOffCommand = wxT("motorcontrol off");
+}
 
+void SessionPage::FixLayout()
+{
+	// Needs to be called AFTER setuphardware!
 	// Adjust the minimum size of the extension and listbox
-
 	wxFont myFont = m_machineList->GetFont();
 	wxMemoryDC * tempDC= new wxMemoryDC();
 	tempDC->SetFont(myFont);
@@ -192,25 +212,14 @@ SessionPage::SessionPage(wxWindow * parent, openMSXController * controller)
 	wMax = dx;
 	items = m_extensionList->GetCount();
 	for (index=0;index<items;index++){
-		tempDC->GetTextExtent(wxString(m_extensionList->GetString(index) +wxT("W")),&w,&h);
+		tempDC->GetTextExtent(wxString(m_extensionList->GetString(index) + wxT("W")),&w,&h); // no idea why we need the extra W...
 		if (w > wMax){
 			wMax = w;
 		}
 	}
-	m_extensionList->SetSizeHints(wMax + wxSystemSettings::GetMetric(wxSYS_HSCROLL_ARROW_X),-1);
+	if (wMax>300) { wMax = 300; } // just to have some limit
+	m_extensionList->SetSizeHints(wMax + wxSystemSettings::GetMetric(wxSYS_VSCROLL_X),-1);
 	delete tempDC;
-
-	RestoreHistory();
-	m_cassettePortState = wxT("disabled");
-	m_romTypeDialog = new RomTypeDlg (wxGetTopLevelParent(this));
-	GetRomTypes();
-	m_ipsDialog = new IPSSelectionDlg (wxGetTopLevelParent(this));
-	m_diskA->control->SetDropTarget(new SessionDropTarget(m_diskA->control));
-	m_diskB->control->SetDropTarget(new SessionDropTarget(m_diskB->control));
-	m_cartA->control->SetDropTarget(new SessionDropTarget(m_cartA->control));
-	m_cartB->control->SetDropTarget(new SessionDropTarget(m_cartB->control));
-	m_cassette->control->SetDropTarget(new SessionDropTarget(m_cassette->control));
-
 }
 
 SessionPage::~SessionPage()
@@ -220,51 +229,71 @@ SessionPage::~SessionPage()
 	delete m_diskB;
 }
 
-void SessionPage::OnClearDiskA(wxCommandEvent &event)
+void SessionPage::OnEjectDiskA(wxCommandEvent &event)
 {
-	m_diskA->control->SetValue(wxT(""));
-	m_diskA->contents = wxT("");
-	m_diskA->ips.Clear();
-	m_diskA->mmenu->SetLabel(Disk_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-	m_lastDiskA = wxT("");
-	m_controller->WriteCommand(wxT("diska eject"));
+	EjectDisk(m_diskA);
 }
 
-void SessionPage::OnClearDiskB(wxCommandEvent &event)
+void SessionPage::OnEjectDiskByMenu(wxCommandEvent & event)
 {
-	m_diskB->control->SetValue(wxT(""));
-	m_diskB->contents = wxT("");
-	m_diskB->ips.Clear();
-	m_diskB->mmenu->SetLabel(Disk_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-	m_lastDiskB = wxT("");
-	m_controller->WriteCommand(wxT("diskb eject"));
+	mediaInfo * target = GetLastMenuTarget();
+	if (target != NULL){
+		EjectDisk(target);
+	}
 }
 
-void SessionPage::OnClearCartA(wxCommandEvent &event)
+void SessionPage::OnEjectDiskB(wxCommandEvent &event)
 {
-	m_cartA->contents = wxT("");
-	m_cartA->ips.Clear();
-	m_cartA->type = wxT("");
-	m_cartA->control->SetValue(wxT(""));
-	m_cartA->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-	m_cartA->mmenu->SetLabel(Cart_Select_Mapper,wxT("Select cartridge type (AUTO)"));
+	EjectDisk(m_diskB);
 }
 
-void SessionPage::OnClearCartB(wxCommandEvent &event)
+void SessionPage::EjectDisk (mediaInfo * target)
 {
-	m_cartB->contents = wxT("");
-	m_cartB->ips.Clear();
-	m_cartB->type = wxT("");
-	m_cartB->control->SetValue(wxT(""));
-	m_cartB->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-	m_cartB->mmenu->SetLabel(Cart_Select_Mapper,wxT("Select cartridge type (AUTO)"));
+	target->control->SetValue(wxT(""));
+	target->control->SetSelection(wxNOT_FOUND);
+	target->contents = wxT("");
+	target->ips.Clear();
+	target->mmenu->SetLabel(Disk_Browse_Ips,wxT("Select IPS Patches (None selected)"));
+	target->lastContents = wxT("");
+	target->oldContents = wxT("");
+	wxString cmd = target->deviceName + wxT(" eject");
+	m_controller->WriteCommand(cmd);
+}
+
+void SessionPage::EjectCart(mediaInfo * target)
+{
+	target->contents = wxT("");
+	target->ips.Clear();
+	target->type = wxT("");
+	target->control->SetValue(wxT(""));
+	target->control->SetSelection(wxNOT_FOUND);
+	target->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
+	target->mmenu->SetLabel(Cart_Select_Mapper,wxT("Select cartridge type (AUTO)"));
+	target->oldContents = wxT("");
+}
+
+void SessionPage::OnEjectCartByMenu (wxCommandEvent & event)
+{
+	mediaInfo * target = GetLastMenuTarget();
+	if (target != NULL){
+		EjectCart(target);
+	}
+}
+void SessionPage::OnEjectCartA(wxCommandEvent &event)
+{
+	EjectCart (m_cartA);
+}
+
+void SessionPage::OnEjectCartB(wxCommandEvent &event)
+{
+	EjectCart (m_cartB);
 }
 
 void SessionPage::OnClearCassette(wxCommandEvent &event)
 {
 	m_cassette->contents = wxT("");
 	m_cassette->control->SetValue(wxT(""));
-	m_lastCassette = wxT("");
+	m_cassette->lastContents = wxT("");
 	m_controller->WriteCommand(wxT("cassetteplayer eject"));
 #ifdef __WXMSW__
 	// Bug in wxMSW? On wxGTK this is not necessary
@@ -275,35 +304,88 @@ void SessionPage::OnClearCassette(wxCommandEvent &event)
 void SessionPage::OnRewind(wxCommandEvent &event)
 {
 	m_controller->WriteCommand(wxT("cassetteplayer rewind"));
+	if (m_recordButton->GetValue()){
+		OnModePlay (event);
+	}
 }
 
-void SessionPage::OnForcePlay(wxCommandEvent &event)
+void SessionPage::OnModePlay (wxCommandEvent & event)
 {
-	if (m_forcePlayButton->GetValue()){
-		m_controller->WriteCommand(wxT("cassetteplayer force_play"));
+	m_playButton->SetValue(true);
+	m_playButton->Enable(false);
+	m_recordButton->Enable(true);
+	m_recordButton->SetValue(false);
+	m_controller->WriteCommand(wxT("cassetteplayer play"));
+}
+
+void SessionPage::OnModeRecord (wxCommandEvent & event)
+{
+	wxString path;
+	wxString tapeImage = wxT("");
+#ifndef __MOTIF__
+	path = wxT("Cassette files (*.wav)|*.wav|All files|*.*||");
+#else
+	path = wxT("*.*");
+#endif
+	bool changeMode = false;
+	if (m_cassetteAutoCreate){
+		changeMode = true;
+	}
+	else{
+		wxFileDialog filedlg(this,wxT("Select Cassettefile to save to"),
+							::wxPathOnly(m_cassette->contents), wxT(""),
+							path ,wxSAVE | wxOVERWRITE_PROMPT);
+		if (filedlg.ShowModal() == wxID_OK){
+			changeMode = true;
+			tapeImage += wxT(" ");
+			tapeImage += ConvertPath(filedlg.GetPath(),true);
+		}
+		else{
+			m_recordButton->SetValue(false);
+		}
+
+	}
+	if (changeMode){
+		m_recordButton->SetValue(true);
+		m_recordButton->Enable(false);
+		m_playButton->Enable(true);
+		m_playButton->SetValue(false);
+		m_controller->WriteCommand(wxT("cassetteplayer new") + tapeImage);
+	}
+}
+
+
+
+void SessionPage::OnMotorControl(wxCommandEvent & event)
+{
+	m_cassetteControl = !m_cassetteControl;
+	if (m_cassetteControl){
+		m_controller->WriteCommand(wxString(wxT("cassetteplayer ")) + m_motorControlOnCommand);
 	}
 	else {
-		m_controller->WriteCommand(wxT("cassetteplayer no_force_play"));
+		m_controller->WriteCommand(wxString(wxT("cassetteplayer ")) + m_motorControlOffCommand);
 	}
 }
 
-void SessionPage::OnForcePlayByMenu(wxCommandEvent & event)
+void SessionPage::OnAutoCassettefile(wxCommandEvent & event)
 {
-	m_forcePlayButton->SetValue(!m_forcePlayButton->GetValue());
-	OnForcePlay(event);
+	m_cassetteAutoCreate = !m_cassetteAutoCreate;
+	ConfigurationData * config = ConfigurationData::instance();
+	config->SetParameter(ConfigurationData::CD_AUTORECORD,(long)(m_cassetteAutoCreate?1:0));
+	config->SaveData();
 }
 
 void SessionPage::OnBrowseDiskA(wxCommandEvent &event)
 {
 	if (BrowseDisk (m_diskA, wxT("diska"), ::wxPathOnly(m_diskA->contents))) {
-		m_lastDiskA = m_diskA->contents;
+		m_diskA->lastContents = m_diskA->contents;
 	}
 }
 
 void SessionPage::OnBrowseDiskB(wxCommandEvent &event)
 {
 	if (BrowseDisk (m_diskB, wxT("diskb"), ::wxPathOnly(m_diskB->contents))) {
-		m_lastDiskB = m_diskB->contents;
+		m_diskB->lastContents = m_diskB->contents;
 	}
 }
 
@@ -394,19 +476,23 @@ void SessionPage::OnBrowseCassette(wxCommandEvent &event)
 				SaveHistory();
 			}
 		}
-		m_lastCassette = m_cassette->contents;
+		m_cassette->lastContents = m_cassette->contents;
 		OnChangeCassetteContents(event);
 	}
 }
 
 void SessionPage::OnClickDiskACombo(wxCommandEvent & event)
 {
+	wxComboBox * box = (wxComboBox *)event.GetEventObject();
+	wxString sel = box->GetString(box->GetSelection());
 	OnClickCombo(event);
-	OnChangeDiskAContents(event);
-	if (m_controller->IsOpenMSXRunning()) {
-		m_controller->WriteCommand(wxT("diska eject"));
-		if (!m_diskA->contents.IsEmpty()) {
-			m_controller->WriteCommand(wxT("diska ") + ConvertPath(m_diskA->contents,true));
+	if (sel != box->GetString(box->GetSelection())) { // HACK to prevent crash
+		OnChangeDiskAContents(event);
+		if (m_controller->IsOpenMSXRunning()) {
+			m_controller->WriteCommand(wxT("diska eject"));
+			if (!m_diskA->contents.IsEmpty()) {
+				m_controller->WriteCommand(wxT("diska ") + ConvertPath(m_diskA->contents,true));
+			}
 		}
 	}
 }
@@ -420,12 +506,16 @@ void SessionPage::OnChangeDiskAContents(wxCommandEvent & event)
 
 void SessionPage::OnClickDiskBCombo(wxCommandEvent & event)
 {
+	wxComboBox * box = (wxComboBox *)event.GetEventObject();
+	wxString sel = box->GetString(box->GetSelection());
 	OnClickCombo(event);
-	OnChangeDiskBContents(event);
-	if (m_controller->IsOpenMSXRunning()) {
-		m_controller->WriteCommand(wxT("diskb eject"));
-		if (!m_diskB->contents.IsEmpty()) {
-			m_controller->WriteCommand(wxT("diskb ") + ConvertPath(m_diskB->contents,true));
+	if (sel != box->GetString(box->GetSelection())) { // HACK to prevent crash
+		OnChangeDiskBContents(event);
+		if (m_controller->IsOpenMSXRunning()) {
+			m_controller->WriteCommand(wxT("diskb eject"));
+			if (!m_diskB->contents.IsEmpty()) {
+				m_controller->WriteCommand(wxT("diskb ") + ConvertPath(m_diskB->contents,true));
+			}
 		}
 	}
 }
@@ -449,23 +539,26 @@ void SessionPage::UpdateMenuMapperLabel(mediaInfo * target)
 void SessionPage::OnClickCartACombo(wxCommandEvent & event)
 {
 	OnClickCombo(event);
-	m_cartA->ips.Clear();
-	m_cartA->contents = m_cartA->control->GetValue();
-	m_cartA->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-	m_cartA->type = m_cartA->typehistory[event.GetInt()];
-	UpdateMenuMapperLabel(m_cartA);
-	m_cartA->avoid_evt = true;
+	if (m_cartA->control->GetValue() != m_cartA->oldContents) { // HACK to prevent crash
+		m_cartA->ips.Clear();
+		m_cartA->contents = m_cartA->control->GetValue();
+		m_cartA->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
+		m_cartA->type = m_cartA->typehistory[event.GetInt()];
+		UpdateMenuMapperLabel(m_cartA);
+		m_cartA->avoid_evt = true;
+		m_cartA->oldContents = m_cartA->contents;
+	}
 }
 
 void SessionPage::OnChangeCartAContents(wxCommandEvent & event)
 {
 	if (!m_cartA->avoid_evt){
-		;
 		m_cartA->ips.Clear();
 		m_cartA->type = wxT("");
 		m_cartA->contents = m_cartA->control->GetValue();
 		m_cartA->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
 		UpdateMenuMapperLabel(m_cartA);
+		m_cartA->oldContents = m_cartA->contents;
 	}
 	m_cartA->avoid_evt = false;
 }
@@ -473,12 +566,15 @@ void SessionPage::OnChangeCartAContents(wxCommandEvent & event)
 void SessionPage::OnClickCartBCombo(wxCommandEvent & event)
 {
 	OnClickCombo(event);
-	m_cartB->ips.Clear();
-	m_cartB->contents = m_cartB->control->GetValue();
-	m_cartB->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-	m_cartB->type = m_cartB->typehistory[event.GetInt()];
-	UpdateMenuMapperLabel(m_cartB);
-	m_cartB->avoid_evt = true;
+	if (m_cartB->control->GetValue() != m_cartB->oldContents) { // HACK to prevent crash
+		m_cartB->ips.Clear();
+		m_cartB->contents = m_cartB->control->GetValue();
+		m_cartB->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
+		m_cartB->type = m_cartB->typehistory[event.GetInt()];
+		UpdateMenuMapperLabel(m_cartB);
+		m_cartB->avoid_evt = true;
+		m_cartB->oldContents = m_cartB->contents;
+	}
 }
 
 void SessionPage::OnChangeCartBContents(wxCommandEvent & event)
@@ -488,7 +584,8 @@ void SessionPage::OnChangeCartBContents(wxCommandEvent & event)
 		m_cartB->type=wxT("");
 		m_cartB->contents = m_cartB->control->GetValue();
 		m_cartB->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-		UpdateMenuMapperLabel(m_cartA);
+		UpdateMenuMapperLabel(m_cartB);
+		m_cartB->oldContents = m_cartB->contents;
 	}
 	m_cartB->avoid_evt = false;
 }
@@ -500,6 +597,11 @@ void SessionPage::OnClickCassetteCombo(wxCommandEvent & event)
 }
 
 void SessionPage::OnChangeCassetteContents(wxCommandEvent &event)
+{
+	HandleCassetteChange();
+}
+
+void SessionPage::HandleCassetteChange ()
 {
 	m_cassette->contents = m_cassette->control->GetValue();
 	if ((m_cassettePortState != wxT("disabled")) &&
@@ -507,16 +609,18 @@ void SessionPage::OnChangeCassetteContents(wxCommandEvent &event)
 		(m_cassette->contents != wxT("")))
 	{
 		m_rewindButton->Enable(true);
-		m_forcePlayButton->Enable(true);
+		m_cassetteControlEnabled = true;
 	}
 	else
 	{
 		m_rewindButton->Enable(false);
-		m_forcePlayButton->Enable(false);
+		m_cassetteControlEnabled = false;
 	}
+	SetCassetteMode (wxT("play"));
+
 }
 
-void SessionPage::SetupHardware (bool initial)
+void SessionPage::SetupHardware (bool initial, bool reset)
 {
 	wxString currentMachine;
 	wxArrayString currentExtensions;
@@ -531,28 +635,52 @@ void SessionPage::SetupHardware (bool initial)
 		}
 	}
 	ConfigurationData * config = ConfigurationData::instance();
-	wxString sharepath;
-	config->GetParameter(ConfigurationData::CD_SHAREPATH,sharepath);
+	m_machineArray.Clear();
+	m_extensionArray.Clear();
 	m_machineList->Clear();
 	m_machineList->Append(wxT(" <default> "));
 	m_extensionList->Clear();
-
-	m_machineArray.Clear();
-	m_extensionArray.Clear();
-
-	prepareExtensions (sharepath, m_extensionArray);
-	prepareMachines (sharepath, m_machineArray);
-	wxString personalShare;
+	wxString checkedMachines;
+	wxString checkedExtensions;
+	config->GetParameter(ConfigurationData::CD_MACHINES, checkedMachines);
+	config->GetParameter(ConfigurationData::CD_EXTENSIONS, checkedExtensions);
+	if (!checkedMachines.IsEmpty() && !checkedExtensions.IsEmpty() && !reset){
+		int pos;
+		do
+		{
+			pos = checkedMachines.Find(wxT("::"));
+			if (pos != -1)
+			{
+				m_machineArray.Add(checkedMachines.Left(pos));
+				checkedMachines = checkedMachines.Mid(pos + 2);
+			}
+		}while (pos !=-1);
+		do
+		{
+			pos = checkedExtensions.Find(wxT("::"));
+			if (pos != -1)
+			{
+				m_extensionArray.Add(checkedExtensions.Left(pos));
+				checkedExtensions = checkedExtensions.Mid(pos + 2);
+			}
+		}while (pos !=-1);
+	}
+	else{
+		wxString sharepath;
+		config->GetParameter(ConfigurationData::CD_SHAREPATH,sharepath);
+		prepareExtensions (sharepath, m_extensionArray);
+		prepareMachines (sharepath, m_machineArray);
+		wxString personalShare;
 #ifdef __UNIX__
-	personalShare = ::wxGetHomeDir() +wxT("/.openMSX/share");
+		personalShare = ::wxGetHomeDir() +wxT("/.openMSX/share");
 #else
-	TCHAR temp[MAX_PATH + 1];
-	SHGetSpecialFolderPath(0,temp,CSIDL_PERSONAL, 1);
-	personalShare = wxString((const wxChar *)temp) +wxT("/openMSX/share");
+		TCHAR temp[MAX_PATH + 1];
+		SHGetSpecialFolderPath(0,temp,CSIDL_PERSONAL, 1);
+		personalShare = wxString((const wxChar *)temp) +wxT("/openMSX/share");
 #endif
-	prepareExtensions (personalShare, m_extensionArray, true);
-	prepareMachines (personalShare, m_machineArray, true);
-
+		prepareExtensions (personalShare, m_extensionArray, true);
+		prepareMachines (personalShare, m_machineArray, true);
+	}
 	m_extensionArray.Sort(SessionPage::CompareCaseInsensitive);
 	m_machineArray.Sort(SessionPage::CompareCaseInsensitive);
 	fillExtensions (m_extensionArray);
@@ -620,6 +748,8 @@ void SessionPage::fillExtensions (wxArrayString & extensionArray)
 
 void SessionPage::prepareMachines(wxString sharepath, wxArrayString & machineArray, bool optional)
 {
+	wxString cmd;
+	ConfigurationData::instance()->GetParameter(ConfigurationData::CD_EXECPATH, cmd);
 	if (!::wxDirExists(sharepath + wxT("/machines"))) {
 		if (!optional) {
 			wxString msg;
@@ -660,35 +790,35 @@ void SessionPage::HandleFocusChange(wxWindow * oldFocus, wxWindow * newFocus)
 	if (m_controller->IsOpenMSXRunning()){
 		if (oldFocus == m_diskA->control) {
 			contents = m_diskA->contents;
-			if (contents != m_lastDiskA) {
+			if (contents != m_diskA->lastContents) {
 				m_controller->WriteCommand(wxT("diska eject"));
 				if (!contents.IsEmpty()) {
 					m_controller->WriteCommand(wxT("diska ") + ConvertPath(m_diskA->contents,true));
 					AddHistory(m_diskA);
 				}
-				m_lastDiskA = m_diskA->contents;
+				m_diskA->lastContents = m_diskA->contents;
 			}
 		}
 		else if (oldFocus == m_diskB->control) {
 			contents = m_diskB->contents;
-			if (contents != m_lastDiskB) {
+			if (contents != m_diskB->lastContents) {
 				m_controller->WriteCommand(wxT("diskb eject"));
 				if (!contents.IsEmpty()) {
 					m_controller->WriteCommand(wxT("diskb ") + ConvertPath(m_diskB->contents,true));
 					AddHistory(m_diskB);
 				}
-				m_lastDiskB = m_diskB->contents;
+				m_diskB->lastContents = m_diskB->contents;
 			}
 		}
 		else if (oldFocus == m_cassette->control) {
-			if (m_cassette->contents != m_lastCassette) {
+			if (m_cassette->contents != m_cassette->lastContents) {
 				contents = m_cassette->contents;
 				m_controller->WriteCommand(wxT("cassetteplayer eject"));
 				if (!contents.IsEmpty()) {
 					m_controller->WriteCommand(wxT("cassetteplayer ") + ConvertPath(m_cassette->contents,true));
 					AddHistory(m_cassette);
 				}
-				m_lastCassette = m_cassette->contents;
+				m_cassette->lastContents = m_cassette->contents;
 			}
 		}
 		SaveHistory();
@@ -733,7 +863,11 @@ void SessionPage::SetControlsOnEnd()
 	m_cartAButton->Enable(true);
 	m_cartBButton->Enable(true);
 	m_rewindButton->Enable(false);
-	m_forcePlayButton->Enable(false);
+	m_playButton->SetValue(false);
+	m_playButton->Enable(false);
+	m_recordButton->SetValue(false);
+	m_recordButton->Enable(false);
+	m_cassetteControlEnabled = false;
 	m_cassetteButton->Enable(true);
 	m_clearCassette->Enable(true);
 	m_browseCassette->Enable(true);
@@ -755,18 +889,18 @@ void SessionPage::SetCassetteControl()
 		(m_cassettePortState != wxT("cassetteplayers"))) {
 		if (m_cassette->contents != wxT("")) {
 			m_rewindButton->Enable(true);
-			m_forcePlayButton->Enable(true);
+			m_cassetteControlEnabled = true;
 		}
 		else{
 			m_rewindButton->Enable(false);
-			m_forcePlayButton->Enable(false);
+			m_cassetteControlEnabled = false;
 		}
 		state = true;
 	}
 	else {
 		state = false;
 		m_rewindButton->Enable(false);
-		m_forcePlayButton->Enable(false);
+		m_cassetteControlEnabled = false;
 	}
 	m_cassetteButton->Enable(state);
 	m_clearCassette->Enable(state);
@@ -845,6 +979,8 @@ void SessionPage::UpdateSessionData()
 	m_InsertedMedia = 0;
 	i=0;
 	FOREACH(i,media){
+		media[i]->contents = media[i]->control->GetValue();
+		media[i]->lastContents = media[i]->contents;
 		if (!media[i]->contents.IsEmpty()) {
 			AddHistory (media[i]);
 			m_InsertedMedia |= flags [i];
@@ -900,7 +1036,6 @@ void SessionPage::AddHistory(mediaInfo *media)
 	for (i=0;i<media->history.GetCount();i++){
 		media->control->Append(media->history[i]);
 	}
-//	media->control->SetValue(currentItem);
 	media->control->SetSelection(0);
 }
 
@@ -950,9 +1085,10 @@ void SessionPage::RestoreHistory()
 				media[i]->typehistory.Add(wxT("auto"));
 			}
 		}
-		if (m_InsertedMedia & (1 << i)) {
+		if ((m_InsertedMedia & (1 << i)) && (media[i]->history.GetCount() >0)) {
 			media[i]->control->SetSelection(0);
 			media[i]->contents=media[i]->history[0];
+			media[i]->oldContents=media[i]->history[0];
 			if ((media[i]==m_cartA) || (media[i]==m_cartB)){
 				media[i]->type=media[i]->typehistory[0];
 				UpdateMenuMapperLabel(media[i]);
@@ -961,10 +1097,13 @@ void SessionPage::RestoreHistory()
 		else {
 			media[i]->control->SetValue(wxT(""));
 			media[i]->contents=wxT("");
+			media[i]->oldContents=wxT("");
 		}
 	}
-		config->GetParameter(ConfigurationData::CD_USEDMACHINE, m_usedMachine);
+	config->GetParameter(ConfigurationData::CD_USEDMACHINE, m_usedMachine);
+	// printf("Last used machine: %s....", (const char*)(wxConvUTF8.cWX2MB(m_usedMachine)));
 	if (!m_usedMachine.IsEmpty()){
+		// printf ("OK, it's not empty\n");
 		temp = m_usedMachine;
 		temp.Replace(wxT("_"),wxT(" "),true);
 		temp.Replace(wxT("\""),wxT(""),true);
@@ -973,6 +1112,10 @@ void SessionPage::RestoreHistory()
 		if (pos != -1){
 			m_machineList->SetSelection (pos);
 		}
+		else {
+			m_machineList->SetSelection (0);
+			// printf(" Can't find the machine\n");
+		}
 	}
 	config->GetParameter(ConfigurationData::CD_USEDEXTENSIONS,value);
 	m_usedExtensions = value;
@@ -982,7 +1125,9 @@ void SessionPage::RestoreHistory()
 		if (pos != -1){
 			temp = value.Left(pos);
 			temp.Replace(wxT("_"),wxT(" "),true);
-			m_extensionList->SetStringSelection (temp);
+			if (m_extensionList->FindString(temp) != -1){
+				m_extensionList->SetStringSelection (temp);
+			}
 			value = value.Mid(pos + 2);
 		}
 	} while (pos !=-1);
@@ -1044,6 +1189,23 @@ void SessionPage::EnableCassettePort (wxString data)
 	m_cassettePortState = data;
 }
 
+void SessionPage::SetCassetteMode (wxString data)
+{
+	bool state = (data == wxT("play"));
+	if (!m_cassette->contents.IsEmpty())
+	{
+		m_playButton->SetValue(state);
+		m_playButton->Enable (!state);
+	}
+	else{
+		m_playButton->SetValue(false);
+		m_playButton->Enable(false);
+	}
+	state = (data == wxT("record"));
+	m_recordButton->SetValue(state);
+	m_recordButton->Enable(!state);
+}
+
 void SessionPage::AutoPlugCassette ()
 {
 	if (m_cassette->contents != wxT("")) {
@@ -1081,8 +1243,9 @@ void SessionPage::OnClickCasMenu (wxCommandEvent & event)
 	m_lastUsedPopup = (wxButton *)event.GetEventObject();
 	wxRect myRect = ((wxWindow *)event.GetEventObject())->GetRect();
 	m_casMenu->Enable(Cas_Rewind,m_rewindButton->IsEnabled());
-	m_casMenu->Enable(Cas_ForcePlay,m_forcePlayButton->IsEnabled());
-	m_casMenu->Check(Cas_ForcePlay,m_forcePlayButton->GetValue());
+	m_casMenu->Enable(Cas_MotorControl,m_cassetteControlEnabled);
+	m_casMenu->Check(Cas_MotorControl,m_cassetteControl);
+	m_casMenu->Check(Cas_AutoCreateFile,m_cassetteAutoCreate);
 	PopupMenu(m_casMenu,myRect.GetLeft(),myRect.GetBottom());
 }
 
@@ -1182,28 +1345,6 @@ void SessionPage::OnBrowseDiskDirByMenu(wxCommandEvent &event)
 	}
 }
 
-void SessionPage::OnEjectDiskByMenu(wxCommandEvent & event)
-{
-	mediaInfo * target = GetLastMenuTarget();
-	if (target != NULL){
-		target->contents = wxT("");
-		target->control->Clear();
-		target->mmenu->SetLabel(Disk_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-	}
-}
-
-void SessionPage::OnEjectCartByMenu (wxCommandEvent & event)
-{
-	mediaInfo * target = GetLastMenuTarget();
-	if (target != NULL){
-		target->contents = wxT("");
-		target->control->Clear();
-		target->ips.Clear();
-		target->mmenu->SetLabel(Cart_Browse_Ips,wxT("Select IPS Patches (None selected)"));
-		target->mmenu->SetLabel(Cart_Select_Mapper,wxT("Select cartridge type (AUTO)"));
-	}
-}
-
 void SessionPage::OnSelectMapper(wxCommandEvent & event)
 {
 	mediaInfo * target = GetLastMenuTarget();
@@ -1269,67 +1410,51 @@ void SessionPage::OnBrowseDiskIps(wxCommandEvent & event)
 	}
 }
 
-void SessionPage::OnCloseMenu(wxMenuEvent & event)
-{
-	wxMessageBox(wxT("closed"));
-}
-
 void SessionPage::GetRomTypes ()
 {
 	// TODO get it from openMSX
+	// NOTE: this list is incomplete! REALLY GET IT FROM OPENMSX!
 	SetupRomType(wxT(""),wxT("Autodetect type"));
 	SetupRomType(wxT("AUTO"),wxT("Autodetect type"));
-	SetupRomType(wxT("PAGE0"),wxT("Plain 16 kB Page 0"));
-	SetupRomType(wxT("PAGE1"),wxT("Plain 16 kB Page 1"));
-	SetupRomType(wxT("PAGE2"),wxT("Plain 16 kB Page 2 (Basic)"));
-	SetupRomType(wxT("ROMBAS"),wxT("Plain 16 kB Page 2 (Basic)"));
-	SetupRomType(wxT("PAGE3"),wxT("Plain 16 kB Page 3"));
-	SetupRomType(wxT("PAGE01"),wxT("Plain 32 kB Page 0-1"));
-	SetupRomType(wxT("PAGE12"),wxT("Plain 32 kB Page 1-2"));
-	SetupRomType(wxT("PAGE23"),wxT("Plain 32 kB Page 2-3"));
-	SetupRomType(wxT("PAGE012"),wxT("Plain 48 kB Page 0-2"));
-	SetupRomType(wxT("PAGE123"),wxT("Plain 48 kB Page 1-3"));
-	SetupRomType(wxT("PAGE0123"),wxT("Plain 64 kB"));
-	SetupRomType(wxT("PLAIN"),wxT("Plain (Any size)"));
-	SetupRomType(wxT("64kB"),wxT("Plain (Any size)"));
-//	SetupRomType(wxT("DRAM"),wxT("DRAM"));
+	SetupRomType(wxT("Page0"),wxT("Plain 16 kB Page 0"));
+	SetupRomType(wxT("Page1"),wxT("Plain 16 kB Page 1"));
+	SetupRomType(wxT("Page2"),wxT("Plain 16 kB Page 2 (Basic)"));
+	SetupRomType(wxT("Normal8000"),wxT("Plain 16 kB Page 2 (Basic)"));
+	SetupRomType(wxT("Page3"),wxT("Plain 16 kB Page 3"));
+	SetupRomType(wxT("Page01"),wxT("Plain 32 kB Page 0-1"));
+	SetupRomType(wxT("Page12"),wxT("Plain 32 kB Page 1-2"));
+	SetupRomType(wxT("Page23"),wxT("Plain 32 kB Page 2-3"));
+	SetupRomType(wxT("Page012"),wxT("Plain 48 kB Page 0-2"));
+	SetupRomType(wxT("Page123"),wxT("Plain 48 kB Page 1-3"));
+	SetupRomType(wxT("Page0123"),wxT("Plain 64 kB"));
+	SetupRomType(wxT("Mirrored"),wxT("Plain (Any size)"));
 	SetupRomType(wxT("8kB"),wxT("Generic 8kB"));
-	SetupRomType(wxT("0"),wxT("Generic 8kB"));
-	SetupRomType(wxT("16kB"),wxT("Generic 16kB"));
-	SetupRomType(wxT("1"),wxT("Generic 16kB"));
-	SetupRomType(wxT("MSXDOS2"),wxT("Generic 16kB"));
-	SetupRomType(wxT("KONAMI4"),wxT("Konami without SCC"));
-	SetupRomType(wxT("3"),wxT("Konami without SCC"));
-	SetupRomType(wxT("KONAMI5"),wxT("Konami with SCC"));
-	SetupRomType(wxT("SCC"),wxT("Konami with SCC"));
-	SetupRomType(wxT("2"),wxT("Konami with SCC"));
+	SetupRomType(wxT("MSXDOS2"),wxT("MSX-DOS 2"));
+	SetupRomType(wxT("Konami"),wxT("Konami without SCC"));
+	SetupRomType(wxT("KonamiSCC"),wxT("Konami with SCC"));
 	SetupRomType(wxT("ASCII8"),wxT("Ascii 8kB"));
-	SetupRomType(wxT("4"),wxT("Ascii 8kB"));
 	SetupRomType(wxT("ASCII16"),wxT("Ascii 16kB"));
-	SetupRomType(wxT("5"),wxT("Ascii 16kB"));
-	SetupRomType(wxT("RTYPE"),wxT("R-Type"));
-	SetupRomType(wxT("CROSSBLAIM"),wxT("Cross Blaim"));
-//	SetupRomType(wxT("PANASONIC"),wxT("Panasonic"));
-//	SetupRomType(wxT("NATIONAL"),wxT("National"));
+	SetupRomType(wxT("Padial8"),wxT("Padial 8kB"));
+	SetupRomType(wxT("Padial16"),wxT("Padial 16kB"));
+	SetupRomType(wxT("R-Type"),wxT("R-Type"));
+	SetupRomType(wxT("CrossBlaim"),wxT("Cross Blaim"));
 	SetupRomType(wxT("MSX-AUDIO"),wxT("MSX-Audio"));
-	SetupRomType(wxT("HARRYFOX"),wxT("Harry Fox"));
-	SetupRomType(wxT("HALNOTE"),wxT("Halnote"));
-	SetupRomType(wxT("KOREAN80IN1"),wxT("Korean Multigame 80 in 1"));
-	SetupRomType(wxT("KOREAN90IN1"),wxT("Korean Multigame (90 in 1)"));
-	SetupRomType(wxT("KOREAN126IN1"),wxT("Korean Multigame (126 in 1)"));
-	SetupRomType(wxT("HOLYQURAN"),wxT("Holy Qu'ran"));
-//	SetupRomType(wxT("FSA1FM1"),wxT("Panasonic FS-A1FM Rom1"));
-//	SetupRomType(wxT("FSA1FM2"),wxT("Panasonic FS-A1FM Rom2"));
-	SetupRomType(wxT("ASCII16-2"),wxT("Ascii 16kB with SRAM"));
-	SetupRomType(wxT("HYDLIDE2"),wxT("Ascii 16kB with SRAM"));
-	SetupRomType(wxT("ASCII8-8"),wxT("Ascii 8kB with SRAM"));
-	SetupRomType(wxT("KOEI-8"),wxT("Koei with 8kB SRAM"));
-	SetupRomType(wxT("KOEI-32"),wxT("Koei with 32kB SRAM"));
+	SetupRomType(wxT("HarryFox"),wxT("Harry Fox"));
+	SetupRomType(wxT("Halnote"),wxT("Halnote"));
+	SetupRomType(wxT("Playball"),wxT("Playball"));
+	SetupRomType(wxT("Zemina80in1"),wxT("Zemina Multigame (80 in 1)"));
+	SetupRomType(wxT("Zemina90in1"),wxT("Zemina Multigame (90 in 1)"));
+	SetupRomType(wxT("Zemina126in1"),wxT("Zemina Multigame (126 in 1)"));
+	SetupRomType(wxT("HolyQuran"),wxT("Holy Qu'ran"));
+	SetupRomType(wxT("ASCII16SRAM2"),wxT("Ascii 16kB with SRAM"));
+	SetupRomType(wxT("ASCII8SRAM8"),wxT("Ascii 8kB with SRAM"));
+	SetupRomType(wxT("KoeiSRAM8"),wxT("Koei with 8kB SRAM"));
+	SetupRomType(wxT("KoeiSRAM32"),wxT("Koei with 32kB SRAM"));
 	SetupRomType(wxT("Wizardry"),wxT("Wizardry"));
-	SetupRomType(wxT("GAMEMASTER2"),wxT("Konami Game Master 2"));
-	SetupRomType(wxT("RC755"),wxT("Konami Game Master 2"));
-	SetupRomType(wxT("SYNTHESIZER"),wxT("Konami Synthesizer"));
-	SetupRomType(wxT("MAJUTSUSHI"),wxT("Hai no Majutsushi"));
+	SetupRomType(wxT("GameMaster2"),wxT("Konami Game Master 2"));
+	SetupRomType(wxT("Synthesizer"),wxT("Konami Synthesizer"));
+	SetupRomType(wxT("Majutsushi"),wxT("Hai no Majutsushi"));
+	SetupRomType(wxT("KeyboardMaster"),wxT("Konami Keyboard Master"));
 }
 
 void SessionPage::AddRomType(wxString type)
@@ -1348,6 +1473,16 @@ void SessionPage::SetupRomType(wxString type, wxString fullname)
 	SetRomTypeFullName(type,fullname);
 }
 
+wxArrayString& SessionPage::GetDetectedMachines ()
+{
+	return m_machineArray;
+}
+
+wxArrayString& SessionPage::GetDetectedExtensions ()
+{
+	return m_extensionArray;
+}
+
 SessionDropTarget::SessionDropTarget(wxComboBox * target)
 {
 	m_target = target;
@@ -1365,3 +1500,10 @@ bool SessionDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& f
 	}
 	return true;
 }
+
+void SessionPage::SetOldStyleCassetteControls()
+{
+	m_casInsertCommand = wxT("");
+	m_motorControlOnCommand = wxT("force_play");
+	m_motorControlOffCommand = wxT("no_force_play");
+}
